from setuptools import setup

setup(
    name='tweetple',
    version='0.13006',
    author='Daniela Pinto Veizaga',
    author_email='danielapintoveizaga@gmail.com',
    description='A python wrapper for users of the Academic Research product track',
    packages=['tweetple'],
    url='https://github.com/dapivei/tweetple',
    zip_safe=False
)
